import typing

from magic_hour.core import (
    AsyncBaseClient,
    RequestOptions,
    SyncBaseClient,
    default_request_options,
    to_encodable,
    type_utils,
)
from magic_hour.types import models, params


class FaceSwapClient:
    def __init__(self, *, base_client: SyncBaseClient):
        self._base_client = base_client

    def create(
        self,
        *,
        assets: params.PostV1FaceSwapBodyAssets,
        end_seconds: float,
        height: int,
        start_seconds: float,
        width: int,
        name: typing.Union[
            typing.Optional[str], type_utils.NotGiven
        ] = type_utils.NOT_GIVEN,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> models.PostV1FaceSwapResponse:
        """
        Face Swap video

        Create a Face Swap video. The estimated frame cost is calculated using 30 FPS. This amount is deducted from your account balance when a video is queued. Once the video is complete, the cost will be updated based on the actual number of frames rendered.

        Get more information about this mode at our [product page](/products/face-swap).


        POST /v1/face-swap

        Args:
            name: The name of video
            assets: Provide the assets for face swap. For video, The `video_source` field determines whether `video_file_path` or `youtube_url` field is used
            end_seconds: The end time of the input video in seconds
            height: The height of the final output video. The maximum height depends on your subscription. Please refer to our [pricing page](https://magichour.ai/pricing) for more details
            start_seconds: The start time of the input video in seconds
            width: The width of the final output video. The maximum width depends on your subscription. Please refer to our [pricing page](https://magichour.ai/pricing) for more details
            request_options: Additional options to customize the HTTP request

        Returns:
            Success

        Raises:
            ApiError: A custom exception class that provides additional context
                for API errors, including the HTTP status code and response body.

        Examples:
        ```py
        client.v1.face_swap.create(
            assets={"image_file_path": "image/id/1234.png", "video_source": "file"},
            end_seconds=15,
            height=960,
            start_seconds=0,
            width=512,
            name="Face Swap video",
        )
        ```

        """
        _json = to_encodable(
            item={
                "name": name,
                "assets": assets,
                "end_seconds": end_seconds,
                "height": height,
                "start_seconds": start_seconds,
                "width": width,
            },
            dump_with=params._SerializerPostV1FaceSwapBody,
        )
        return self._base_client.request(
            method="POST",
            path="/v1/face-swap",
            auth_names=["bearerAuth"],
            json=_json,
            cast_to=models.PostV1FaceSwapResponse,
            request_options=request_options or default_request_options(),
        )


class AsyncFaceSwapClient:
    def __init__(self, *, base_client: AsyncBaseClient):
        self._base_client = base_client

    async def create(
        self,
        *,
        assets: params.PostV1FaceSwapBodyAssets,
        end_seconds: float,
        height: int,
        start_seconds: float,
        width: int,
        name: typing.Union[
            typing.Optional[str], type_utils.NotGiven
        ] = type_utils.NOT_GIVEN,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> models.PostV1FaceSwapResponse:
        """
        Face Swap video

        Create a Face Swap video. The estimated frame cost is calculated using 30 FPS. This amount is deducted from your account balance when a video is queued. Once the video is complete, the cost will be updated based on the actual number of frames rendered.

        Get more information about this mode at our [product page](/products/face-swap).


        POST /v1/face-swap

        Args:
            name: The name of video
            assets: Provide the assets for face swap. For video, The `video_source` field determines whether `video_file_path` or `youtube_url` field is used
            end_seconds: The end time of the input video in seconds
            height: The height of the final output video. The maximum height depends on your subscription. Please refer to our [pricing page](https://magichour.ai/pricing) for more details
            start_seconds: The start time of the input video in seconds
            width: The width of the final output video. The maximum width depends on your subscription. Please refer to our [pricing page](https://magichour.ai/pricing) for more details
            request_options: Additional options to customize the HTTP request

        Returns:
            Success

        Raises:
            ApiError: A custom exception class that provides additional context
                for API errors, including the HTTP status code and response body.

        Examples:
        ```py
        await client.v1.face_swap.create(
            assets={"image_file_path": "image/id/1234.png", "video_source": "file"},
            end_seconds=15,
            height=960,
            start_seconds=0,
            width=512,
            name="Face Swap video",
        )
        ```

        """
        _json = to_encodable(
            item={
                "name": name,
                "assets": assets,
                "end_seconds": end_seconds,
                "height": height,
                "start_seconds": start_seconds,
                "width": width,
            },
            dump_with=params._SerializerPostV1FaceSwapBody,
        )
        return await self._base_client.request(
            method="POST",
            path="/v1/face-swap",
            auth_names=["bearerAuth"],
            json=_json,
            cast_to=models.PostV1FaceSwapResponse,
            request_options=request_options or default_request_options(),
        )



import typing

from magic_hour.core import (
    AsyncBaseClient,
    RequestOptions,
    SyncBaseClient,
    default_request_options,
    to_encodable,
    type_utils,
)
from magic_hour.types import models, params


class AiImageUpscalerClient:
    def __init__(self, *, base_client: SyncBaseClient):
        self._base_client = base_client

    def create(
        self,
        *,
        assets: params.PostV1AiImageUpscalerBodyAssets,
        scale_factor: float,
        style: params.PostV1AiImageUpscalerBodyStyle,
        name: typing.Union[
            typing.Optional[str], type_utils.NotGiven
        ] = type_utils.NOT_GIVEN,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> models.PostV1AiImageUpscalerResponse:
        """
        AI Image Upscaler

        Upscale your image using AI. Each 2x upscale costs 50 frames, and 4x upscale costs 200 frames.

        POST /v1/ai-image-upscaler

        Args:
            name: The name of image
            assets: Provide the assets for upscaling
            scale_factor: float
            style: PostV1AiImageUpscalerBodyStyle
            request_options: Additional options to customize the HTTP request

        Returns:
            Success

        Raises:
            ApiError: A custom exception class that provides additional context
                for API errors, including the HTTP status code and response body.

        Examples:
        ```py
        client.v1.ai_image_upscaler.create(
            assets={"image_file_path": "image/id/1234.png"},
            scale_factor=123.0,
            style={"enhancement": "Balanced"},
            name="Image Upscaler image",
        )
        ```

        """
        _json = to_encodable(
            item={
                "name": name,
                "assets": assets,
                "scale_factor": scale_factor,
                "style": style,
            },
            dump_with=params._SerializerPostV1AiImageUpscalerBody,
        )
        return self._base_client.request(
            method="POST",
            path="/v1/ai-image-upscaler",
            auth_names=["bearerAuth"],
            json=_json,
            cast_to=models.PostV1AiImageUpscalerResponse,
            request_options=request_options or default_request_options(),
        )


class AsyncAiImageUpscalerClient:
    def __init__(self, *, base_client: AsyncBaseClient):
        self._base_client = base_client

    async def create(
        self,
        *,
        assets: params.PostV1AiImageUpscalerBodyAssets,
        scale_factor: float,
        style: params.PostV1AiImageUpscalerBodyStyle,
        name: typing.Union[
            typing.Optional[str], type_utils.NotGiven
        ] = type_utils.NOT_GIVEN,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> models.PostV1AiImageUpscalerResponse:
        """
        AI Image Upscaler

        Upscale your image using AI. Each 2x upscale costs 50 frames, and 4x upscale costs 200 frames.

        POST /v1/ai-image-upscaler

        Args:
            name: The name of image
            assets: Provide the assets for upscaling
            scale_factor: float
            style: PostV1AiImageUpscalerBodyStyle
            request_options: Additional options to customize the HTTP request

        Returns:
            Success

        Raises:
            ApiError: A custom exception class that provides additional context
                for API errors, including the HTTP status code and response body.

        Examples:
        ```py
        await client.v1.ai_image_upscaler.create(
            assets={"image_file_path": "image/id/1234.png"},
            scale_factor=123.0,
            style={"enhancement": "Balanced"},
            name="Image Upscaler image",
        )
        ```

        """
        _json = to_encodable(
            item={
                "name": name,
                "assets": assets,
                "scale_factor": scale_factor,
                "style": style,
            },
            dump_with=params._SerializerPostV1AiImageUpscalerBody,
        )
        return await self._base_client.request(
            method="POST",
            path="/v1/ai-image-upscaler",
            auth_names=["bearerAuth"],
            json=_json,
            cast_to=models.PostV1AiImageUpscalerResponse,
            request_options=request_options or default_request_options(),
        )



import typing


class ApiError(Exception):
    """
    A custom exception class for handling API-related errors.

    This class extends the base Exception class to provide additional context
    for API errors, including the HTTP status code and response body.

    Attributes:
        status_code: The HTTP status code associated with the error.
            None if no status code is applicable.
        body: The response body or error message content.
            Can be any type depending on the API response format.
    """

    status_code: typing.Optional[int]
    body: typing.Any

    def __init__(
        self, *, status_code: typing.Optional[int] = None, body: typing.Any = None
    ) -> None:
        """
        Initialize the ApiError with optional status code and body.

        Args:
            status_code: The HTTP status code of the error.
                Defaults to None.
            body: The response body or error message content.
                Defaults to None.

        Note:
            The asterisk (*) in the parameters forces keyword arguments,
            making the instantiation more explicit.
        """
        self.status_code = status_code
        self.body = body

    def __str__(self) -> str:
        """
        Return a string representation of the ApiError.

        Returns:
            str: A formatted string containing the status code and body.
                Format: "status_code: {status_code}, body: {body}"
        """
        return f"status_code: {self.status_code}, body: {self.body}"



import typing
import typing_extensions
import pydantic


class PostV1AnimationBodyAssets(typing_extensions.TypedDict):
    """
    Provide the assets for animation.
    """

    audio_file_path: typing_extensions.NotRequired[str]
    """
    The path of the input audio. This is the `file_path` field from the response of the [upload urls API](/docs/api/tag/files/post/v1/files/upload-urls). This field is required if `audio_source` is `file`
    """

    audio_source: typing_extensions.Required[
        typing_extensions.Literal["file", "none", "youtube"]
    ]
    """
    Optionally add an audio source if you'd like to incorporate audio into your video
    """

    image_file_path: typing_extensions.NotRequired[str]
    """
    An initial image to use a the first frame of the video. This is the `file_path` field from the response of the [upload urls API](/docs/api/tag/files/post/v1/files/upload-urls)
    """

    youtube_url: typing_extensions.NotRequired[str]
    """
    Using a youtube video as the input source. This field is required if `audio_source` is `youtube`
    """


class _SerializerPostV1AnimationBodyAssets(pydantic.BaseModel):
    """
    Serializer for PostV1AnimationBodyAssets handling case conversions
    and file omissions as dictated by the API
    """

    model_config = pydantic.ConfigDict(
        populate_by_name=True,
    )

    audio_file_path: typing.Optional[str] = pydantic.Field(
        alias="audio_file_path", default=None
    )
    audio_source: typing_extensions.Literal["file", "none", "youtube"] = pydantic.Field(
        alias="audio_source",
    )
    image_file_path: typing.Optional[str] = pydantic.Field(
        alias="image_file_path", default=None
    )
    youtube_url: typing.Optional[str] = pydantic.Field(
        alias="youtube_url", default=None
    )
